%% clear
clear;
close all;
% close all;
format compact;
format short g;

%% inital stuff
prob.theta(1).fun = @(mu) 1./mu(4,:)./(1-mu(2,:).^2);
prob.theta(2).fun = @(mu) mu(2,:)./(1-mu(2,:).^2);
prob.theta(3).fun = @(mu) mu(3,:).*mu(4,:);
prob.theta(4).fun = @(mu) mu(3,:);
prob.theta(5).fun = @(mu) mu(3,:)./mu(4,:);
prob.theta(6).fun = @(mu) mu(1,:).*mu(4,:)./(1-mu(2,:).^2);

prob.D = [0.05 1; ...
          0.1 0.1; ...
          0.02 0.5; ...
          0.1 1];

prob.muRef = [1; 0.1; 0.5; 1];
thetaRef = arrayfun(@(x) x.fun(prob.muRef),prob.theta,'UniformOutput',false);

fem = getInitFem(thetaRef);
prob.fem.dof = getDof(fem);

% pre assembling of equation
prob = performPreAssembling(fem,prob);
   
% compute matrices for inner products
[K,C,D] = assemble(fem,'Out',{'K','C','D'});

L2 = D;
L2 = (L2+L2')/2;
H1 = C+D;
H1 = (H1+H1')/2;
En = (K+K')/2;

ind = prob.fem.dof.ind;
lambdaMin = eigs(En(ind,ind),L2(ind,ind),1,'sm');
En = En + lambdaMin*L2;

prob.fem.innProdFull = struct('L2',L2,'H1',H1,'En',En);
prob.fem.innProd = struct('L2',L2(ind,ind),'H1',H1(ind,ind),'En',En(ind,ind));

%% scm offline
rand('twister',5489);
% random test samle 
scm.n_train = 1500;
scm.Xi_train = repmat(prob.D(:,1),1,scm.n_train) + ...
  repmat(diff(prob.D,[],2),1,scm.n_train).*rand(size(prob.D,1),scm.n_train);
% parameters for scm algorihtm 
scm.M_alpha = inf;
scm.eps_alpha = .75;
scm.Norm = 'En';
scm.K_max = 100;

prob.scm = scm_offline(prob,scm);
clear scm;
save(sprintf('scm_offline_M=%d_ntrain=%d.mat',prob.scm.M_alpha,prob.scm.n_train));

%% scm online
n_test = 100;
Xi_test = repmat(prob.D(:,1),1,n_test) + ...
  repmat(diff(prob.D,[],2),1,n_test).*rand(size(prob.D,1),n_test);
alpha_lb = zeros(n_test,1);
alpha_ub = zeros(n_test,1);
t = zeros(n_test,1);

for n=1:n_test
  tic;
  [alpha_lb(n),alpha_ub(n)] = scm_online(prob,Xi_test(:,n));
  t(n) = toc;
end

[min(t) mean(t) max(t)]

[sort_lb,ind] = sort(alpha_lb);
figure;
plot([sort_lb,alpha_ub(ind)]);
figure;
plot((alpha_ub-alpha_lb)./alpha_ub);